@@ -560,12 +560,16 @@ set_property (GObject * obj, guint id, const GValue * value, GParamSpec * pspec)
 
 	switch (id) {
 	case PROP_DBUSNAME:
+		g_return_if_fail(g_dbus_is_name(g_value_get_string(value)));
+
 		priv->dbus_name = g_value_dup_string(value);
 		if (priv->dbus_name != NULL && priv->dbus_object != NULL) {
 			build_proxies(DBUSMENU_CLIENT(obj));
 		}
 		break;
 	case PROP_DBUSOBJECT:
+		g_return_if_fail(g_variant_is_object_path(g_value_get_string(value)));
+
 		priv->dbus_object = g_value_dup_string(value);
 		if (priv->dbus_name != NULL && priv->dbus_object != NULL) {
 			build_proxies(DBUSMENU_CLIENT(obj));
@@ -2160,8 +2164,29 @@ parse_layout_xml(DbusmenuClient * client, GVariant * layout, DbusmenuMenuitem *
 		for (childsearch = oldchildren; childsearch != NULL; childsearch = g_list_next(childsearch)) {
 			DbusmenuMenuitem * cs_mi = DBUSMENU_MENUITEM(childsearch->data);
 			if (childid == dbusmenu_menuitem_get_id(cs_mi)) {
-				oldchildren = g_list_remove(oldchildren, cs_mi);
-				childmi = cs_mi;
+				GVariantIter iter;
+				gchar * prop;
+				GVariant * value;
+				GVariant * child_props;
+				GVariant * new_type = NULL;
+				GVariant * old_type = NULL;
+
+				child_props = g_variant_get_child_value(child, 1);
+				g_variant_iter_init(&iter, child_props);
+				while (g_variant_iter_loop(&iter, "{sv}", &prop, &value)) {
+					if (g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_TYPE) == 0) {
+						new_type = value;
+						break;
+					}
+				}
+				g_variant_unref(child_props);
+				
+				old_type = dbusmenu_menuitem_property_get_variant(cs_mi, DBUSMENU_MENUITEM_PROP_TYPE);
+				if ((old_type == NULL && new_type == NULL) || (old_type != NULL && new_type != NULL && g_variant_compare(old_type, new_type) == 0)) {
+					// Only recycle the menu item if it's of the same type
+					oldchildren = g_list_remove(oldchildren, cs_mi);
+					childmi = cs_mi;
+				}
 				break;
 			}
 		}
@@ -2451,6 +2476,9 @@ update_layout (DbusmenuClient * client)
 DbusmenuClient *
 dbusmenu_client_new (const gchar * name, const gchar * object)
 {
+	g_return_val_if_fail(g_dbus_is_name(name), NULL);
+	g_return_val_if_fail(g_variant_is_object_path(object), NULL);
+
 	DbusmenuClient * self = g_object_new(DBUSMENU_TYPE_CLIENT,
 	                                     DBUSMENU_CLIENT_PROP_DBUS_NAME, name,
 	                                     DBUSMENU_CLIENT_PROP_DBUS_OBJECT, object,
@@ -2511,7 +2539,7 @@ type_handler_destroy (gpointer user_data)
  *     items when they come in.
  * 
  * This function connects into the type handling of the #DbusmenuClient.
- * Every new menuitem that comes in immediately gets asked for it's
+ * Every new menuitem that comes in immediately gets asked for its
  * properties.  When we get those properties we check the 'type'
  * property and look to see if it matches a handler that is known
  * by the client.  If so, the @newfunc function is executed on that
@@ -2542,7 +2570,7 @@ dbusmenu_client_add_type_handler (DbusmenuClient * client, const gchar * type, D
  * 	the resources in @user_data.
  * 
  * This function connects into the type handling of the #DbusmenuClient.
- * Every new menuitem that comes in immediately gets asked for it's
+ * Every new menuitem that comes in immediately gets asked for its
  * properties.  When we get those properties we check the 'type'
  * property and look to see if it matches a handler that is known
  * by the client.  If so, the @newfunc function is executed on that
