@@ -97,9 +97,7 @@ static void           a11y_name_notify_cb      (AtkObject *         accessible,
                                                 gpointer            data);
 static void           item_inserted_cb         (GtkContainer *      menu,
                                                 GtkWidget *         widget,
-#ifdef HAVE_GTK3
                                                 gint                position,
-#endif
                                                 gpointer            data);
 static void           item_removed_cb          (GtkContainer *      menu,
                                                 GtkWidget *         widget,
@@ -273,10 +271,47 @@ dbusmenu_gtk_parse_menu_structure (GtkWidget * widget)
 DbusmenuMenuitem *
 dbusmenu_gtk_parse_get_cached_item (GtkWidget * widget)
 {
-	if (!GTK_IS_MENU_ITEM(widget)) {
-		return NULL;
-	}
-	return DBUSMENU_MENUITEM(g_object_get_data(G_OBJECT(widget), CACHED_MENUITEM));
+  GObject * o = NULL;
+  DbusmenuMenuitem * ret = NULL;
+
+  if (GTK_IS_MENU_ITEM (widget))
+    o = g_object_get_data (G_OBJECT(widget), CACHED_MENUITEM);
+ 
+  if (o && DBUSMENU_IS_MENUITEM(o))
+    ret = DBUSMENU_MENUITEM (o);
+
+  return ret;
+}
+
+/* remove our dbusmenuitem's hooks to a GtkWidget,
+   such as when either of them are being destroyed */
+static void
+disconnect_from_widget (GtkWidget * widget)
+{
+  ParserData * pdata = parser_data_get_from_widget (widget);
+
+  if (pdata && pdata->widget)
+    {
+      GObject * o;
+
+      g_assert (pdata->widget == widget);
+
+      /* stop listening to signals from the widget */
+      o = G_OBJECT (pdata->widget);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_notify_handler_id);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_add_handler_id);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_accel_handler_id);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_toggle_handler_id);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_visible_handler_id);
+      dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_screen_changed_handler_id);
+
+      /* clear the menuitem's widget pointer */
+      g_object_remove_weak_pointer (o, (gpointer*)&pdata->widget);
+      pdata->widget = NULL;
+
+      /* clear the widget's menuitem pointer */
+      g_object_set_data(o, CACHED_MENUITEM, NULL);
+    }
 }
 
 static void
@@ -297,17 +332,7 @@ parser_data_free (ParserData * pdata)
 	}
 
 	if (pdata->widget != NULL) {
-		GObject * o = G_OBJECT(pdata->widget);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_notify_handler_id);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_add_handler_id);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_accel_handler_id);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_toggle_handler_id);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_visible_handler_id);
-		dbusmenu_gtk_clear_signal_handler (o, &pdata->widget_screen_changed_handler_id);
-		g_object_remove_weak_pointer(o, (gpointer*)&pdata->widget);
-
-		/* since the DbusmenuMenuitem is being destroyed, uncache it from the GtkWidget */
-		g_object_steal_data(o, CACHED_MENUITEM);
+		disconnect_from_widget (pdata->widget);
 	}
 
 	if (pdata->settings != NULL) {
@@ -340,7 +365,7 @@ parser_data_free (ParserData * pdata)
 	return;
 }
 
-/* Gets the positon of the child with its' parent if it has one.
+/* Gets the positon of the child with its parent if it has one.
    Returns -1 if the position is unable to be calculated. */
 static gint
 get_child_position (GtkWidget * child)
@@ -379,7 +404,7 @@ new_menuitem (GtkWidget * widget)
 
 	pdata->widget = widget;
 	g_object_add_weak_pointer(G_OBJECT (widget), (gpointer*)&pdata->widget);
-	g_object_set_data(G_OBJECT(widget), CACHED_MENUITEM, item);
+	g_object_set_data_full(G_OBJECT(widget), CACHED_MENUITEM, g_object_ref(item), g_object_unref);
 
 	return item;
 }
@@ -404,11 +429,7 @@ watch_submenu(DbusmenuMenuitem * mi, GtkWidget * menu)
 
 	pdata->shell = menu;
 	pdata->item_inserted_handler_id = g_signal_connect (G_OBJECT (menu),
-#ifdef HAVE_GTK3
                           "insert",
-#else
-                          "child-added",
-#endif
 		          G_CALLBACK (item_inserted_cb),
 		          mi);
 	pdata->item_removed_handler_id = g_signal_connect (G_OBJECT (menu),
@@ -1121,6 +1142,14 @@ item_activated (DbusmenuMenuitem *item, guint timestamp, gpointer user_data)
 
       if (GTK_IS_MENU_ITEM (child))
         {
+          GtkWidget *parent = gtk_widget_get_parent(child);
+          if (GTK_IS_MENU (parent))
+            {
+              gint pos = get_child_position (child);
+              if (pos >= 0)
+                gtk_menu_set_active (GTK_MENU(parent), pos);
+            }
+
           gdk_threads_enter ();
           gtk_menu_item_activate (GTK_MENU_ITEM (child));
           gdk_threads_leave ();
@@ -1359,9 +1388,7 @@ settings_notify_cb (GtkSettings * settings, GParamSpec * pspec, gpointer data)
 static void
 item_inserted_cb (GtkContainer *menu,
                   GtkWidget    *widget,
-#ifdef HAVE_GTK3
                   gint          position,
-#endif
                   gpointer      data)
 {
 	DbusmenuMenuitem *menuitem = (DbusmenuMenuitem *)data;
@@ -1379,24 +1406,19 @@ item_inserted_cb (GtkContainer *menu,
 
 /* A child item was removed from a menu we're watching. */
 static void
-item_removed_cb (GtkContainer *menu, GtkWidget *widget, gpointer data)
+item_removed_cb (GtkContainer *parent_w, GtkWidget *child_w, gpointer data)
 {
-	gpointer pmi = g_object_get_data(G_OBJECT(widget), CACHED_MENUITEM);
-	if (pmi == NULL) {
-		return;
-	}
+  DbusmenuMenuitem * child_mi;
 
-	DbusmenuMenuitem * child = DBUSMENU_MENUITEM(pmi);
+  if ((child_mi = dbusmenu_gtk_parse_get_cached_item (child_w)))
+    {
+      DbusmenuMenuitem * parent_mi;
 
-	pmi = g_object_get_data(G_OBJECT(menu), CACHED_MENUITEM);
-	if (pmi == NULL) {
-		return;
-	}
+      if ((parent_mi = dbusmenu_gtk_parse_get_cached_item (GTK_WIDGET(parent_w))))
+        dbusmenu_menuitem_child_delete (parent_mi, child_mi); 
 
-	DbusmenuMenuitem * parent = DBUSMENU_MENUITEM(pmi);
-
-	dbusmenu_menuitem_child_delete(parent, child);
-	return;
+      disconnect_from_widget (child_w);
+    }
 }
 
 static gboolean
